@@ -356,6 +356,9 @@ getMetaPrograms <- function(nmf.res,
 #' @param mp.res The meta-programs object generated by \code{\link{getMetaPrograms}}
 #' @param similarity.cutoff Min and max values for similarity metric
 #' @param scale Heatmap rescaling (passed to pheatmap as 'scale')
+#' @param downsample Limit max number of samples in heatmap, to avoid overloading
+#'     the graphics
+#' @param showtree Whether to plot the hierarchical clustering tree
 #' @param palette Heatmap color palette (passed to pheatmap as 'color')
 #' @param annotation_colors Color palette for MP annotations 
 #' @param main Heatmap title 
@@ -378,6 +381,8 @@ getMetaPrograms <- function(nmf.res,
 plotMetaPrograms <- function(mp.res,
                             similarity.cutoff=c(0,1),
                             scale = "none",
+                            downsample = 1000,
+                            showtree = TRUE,
                             palette = viridis(100, option="A", direction=-1),
                             annotation_colors = NULL,
                             main = "Clustered Heatmap",
@@ -388,16 +393,32 @@ plotMetaPrograms <- function(mp.res,
   J <- mp.res[["programs.similarity"]]
   tree <- mp.res[["programs.tree"]]
   cl_members <- mp.res[["programs.clusters"]]
-
+  labs.order <- labels(as.dendrogram(tree))
+  
+  #downsample, to avoid overloading the graphics
+  if (length(cl_members) > downsample) {
+    sid.keep <- downsampleMin(cl_members, size = downsample)
+    labs.order <- labs.order[labs.order %in% sid.keep]
+    
+    cl_members <- cl_members[labs.order]
+    J <- J[labs.order, labs.order]
+    
+    #disable 'tree' for downsampled heatmap
+    showtree <- FALSE
+  }
+  
   cl_names <- names(cl_members)
   cl_members[!is.na(cl_members)] <- paste0("MP",cl_members[!is.na(cl_members)])
   names(cl_members) <- cl_names
 
   #Recover order of MP clusters
-  labs.order <- labels(as.dendrogram(tree))
   cluster.order <- unique(cl_members[labs.order])
   nMP <- length(cluster.order)
   
+  #Gaps for heatmap
+  diffs <- diff(as.integer(as.factor(cl_members))) 
+  gaps <- which(diffs != 0)
+  
   #Annotation column
   annotation_col <- as.data.frame(cl_members)
   colnames(annotation_col) <- "Metaprogram"
@@ -407,6 +428,10 @@ plotMetaPrograms <- function(mp.res,
   J[J<similarity.cutoff[1]] <- similarity.cutoff[1]
   J[J>similarity.cutoff[2]] <- similarity.cutoff[2]
   
+  if (!showtree) {
+    tree <- FALSE
+  }
+  
   ph <- pheatmap(J,
                  scale = scale,
                  color = palette,
@@ -415,6 +440,8 @@ plotMetaPrograms <- function(mp.res,
                  cluster_cols = tree,
                  cutree_rows = nMP,
                  cutree_cols = nMP,
+                 gaps_row = gaps,
+                 gaps_col = gaps,
                  annotation_col = annotation_col,
                  annotation_row = annotation_col,
                  annotation_colors = annotation_colors,
@@ -644,3 +671,62 @@ findVariableFeatures_wfilters <- function(
   return(obj)
 }  
 
+#' Drop meta-programs
+#'
+#' Remove meta-programs from the GeneNMF results. This can be desirable if
+#' one or more MPs are redundant or have poor metrics (e.g. low sample coverage
+#' or low silhouette score).
+#'
+#' @param mp.res The meta-programs object generated by \code{\link{getMetaPrograms}}
+#' @param dropMP A vector with the names of the MPs to be dropped
+#' @return The input meta-program object, minus the dropped MPs
+#'
+#' @examples
+#' library(Seurat)
+#' data(sampleObj)
+#' geneNMF_programs <- multiNMF(list(sampleObj), k=5)
+#' geneNMF_metaprograms <- getMetaPrograms(geneNMF_programs, nMP=3)
+#' geneNMF_metaprograms_filtered <- dropMetaPrograms(geneNMF_metaprograms, dropMP = c("MP2")) 
+#' 
+#' @importFrom dendextend prune
+#' @importFrom stats as.hclust as.dendrogram
+#' @export  
+dropMetaPrograms <- function(mp.res,
+                             dropMP=NULL) {
+ 
+  mp.all <- names(mp.res$metaprograms.genes)
+  todrop <- as.vector(dropMP)
+  
+  missing <- setdiff(todrop, mp.all)
+  keep <- setdiff(mp.all, todrop)
+  if (length(missing) > 0) {
+    message <- paste(missing, collapse = ",")
+    message <- paste0("One or more MPs not found in object: ", message)
+    warning(message)
+  }
+  
+  #drop gene lists
+  mp.res$metaprograms.genes[todrop] <- NULL
+  mp.res$metaprograms.genes.weights[todrop] <- NULL
+  #drop from metrics
+  mp.res$metaprograms.metrics <- mp.res$metaprograms.metrics[keep, ]
+  
+  #drop from the tree (prune)
+  tree <- mp.res$programs.tree
+  which.drop <- paste0("MP", mp.res$programs.clusters) %in% c(todrop,"MPNA")
+  drop.row <- mp.res$programs.clusters[which.drop]
+  
+  dend <- prune(as.dendrogram(tree), names(drop.row))
+  
+  #drop from the clusters
+  mp.res$programs.clusters <- mp.res$programs.clusters[!names(mp.res$programs.clusters) %in% names(drop.row)]
+  #drop from similarity matrix
+  pk <- names(mp.res$programs.clusters)
+  mp.res$programs.similarity <- mp.res$programs.similarity[pk, pk]
+  
+  mp.res$programs.tree <- as.hclust(dend)
+  mp.res$programs.tree$method <- tree$method
+  mp.res$programs.tree$dist.method <- tree$dist.method
+  mp.res$programs.tree$call <- tree$call
+  return(mp.res)
+}  
